$tenantGuid 	= "Set-Tenant-Guid-Here"
$eidBaseUrl 	= "https://idp.blackberry.com"
$clientId 		= "Set-Client-Id-Here";
$clientSecret 	= "Set-Client-Secret-Here";
$scope 			= "MDMBWS.All"
# UEM On-premise API port is 18084 and Cloud API port is 443
$restApiUrl 	= "https://Set-Host-Here:18084/{0}/api/v1/ping" -f $tenantGuid;

function main
{
    $methodName = "Main()";
    enterMethod($methodName);

    # Get Discovery document for tenant
    try
    {
        [Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Ssl3, [Net.SecurityProtocolType]::Tls, [Net.SecurityProtocolType]::Tls11, [Net.SecurityProtocolType]::Tls12

        $discoveryUrl = $eidBaseUrl + "/op/tenant/{0}/.well-known/openid-configuration" -f $tenantGuid;

        Write-Output ""
        Write-Output "Getting Discovery Document from $discoveryUrl"

        $result = Invoke-RestMethod -Method GET -Uri $discoveryUrl -ErrorAction STOP

        if ($result.token_endpoint){
            $tokenEndpoint = $result.token_endpoint
            Write-Output ""
            Write-Output "Obtained token endpoint: $tokenEndpoint"
        }
    }
    catch
    {
        handleError $_;
    }
    
    # Get access token from Identity Provider
    try
    {
        $encodedSecret = [uri]::EscapeDataString($clientSecret) 
        $basicHeader = @{"Authorization" = ("Basic",[Convert]::ToBase64String([Text.Encoding]::ASCII.GetBytes(($clientId, $encodedSecret -join ":"))) -join " ")}

        $tokenRequest = @{grant_type="client_credentials"; scope=$scope}
  
        Write-Output ""
        Write-Output "Attempting to get an access token with client_credentials and client secret"
        Write-Output "Request uri: $tokenEndpoint"

        $result = Invoke-RestMethod -Method Post -Uri $tokenEndpoint  -Body $tokenRequest -ContentType "application/x-www-form-urlencoded" -Headers $basicHeader -ErrorAction STOP

        if ($result.access_token){
            $global:accessToken = $result.access_token
            Write-Output "Received access token:"
            $accessToken
        }
    }
    catch
    {
        handleError($_);
    }

    # Invoke the REST Api with the access token in the Authorization header
    try
    {
        # Add Trust All
        add-type @"
        using System.Net;
        using System.Security.Cryptography.X509Certificates;
        public class TrustAllCertsPolicy : ICertificatePolicy {
            public bool CheckValidationResult(
                ServicePoint srvPoint, X509Certificate certificate,
                WebRequest request, int certificateProblem) {
                return true;
            }
        }
"@
        [System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy
        [Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Ssl3, [Net.SecurityProtocolType]::Tls, [Net.SecurityProtocolType]::Tls11, [Net.SecurityProtocolType]::Tls12

        $authHeader = @{"Authorization" = ("Bearer", $accessToken -join " ")}

        Write-Output ""
        Write-Output "Attempting to access Public REST API"
        Write-Output "Request: $restApiUrl"

        $response = Invoke-RestMethod -Method GET -Uri $restApiUrl -Headers $authHeader -ErrorAction STOP

        Write-Output ""
        Write-Output "Received response for rest api: $restApiUrl"
        Write-Output $response
        Write-Output ""
    }
    catch
    {
        handleError $_;
    }

    Write-Output ""
    exitMethod($methodName);
    Write-Output ""
    Write-Output("Complete!");
    exit(0);
}

<#
<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command could not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

# Call main function.
main;